-- 
-- Anlegen eines WASKA-Datenbankclusters unter PostgreSQL
--
-- Autor: Frank Koormann <frank@intevation.de>
-- Stand: 2008-01-09
--
-- Notwendige Anpassungen: 
--

-- TODO:
-- - Lschen einer Datenbank.
-- 

\connect postgres postgres

-- ------------------------------------------------------------------------
-- Anlegen der Basisrolen und Wartungsdatenbank
--

-- Generische Rolle WASKA-Admin, Verwaltungsrollen erben diese Rolle, um 
-- Zugang zur Wartungsdatenbank zu bekommen.
-- 01.11.2007: Brauchen wir die noch?
CREATE ROLE waska_admin NOLOGIN
    NOSUPERUSER NOINHERIT NOCREATEDB NOCREATEROLE;

-- Eine extra Rolle, die Rollen/Benutzer anlegen/verwalten darf. Diese Rolle
-- definiert spter die Funktionen zur Verwaltung der Benutzer, die ber
-- "SECURITY DEFINER" dann mit erhhten Rechten laufen.
CREATE ROLE waska_role_adm NOLOGIN
    NOSUPERUSER NOINHERIT NOCREATEDB CREATEROLE;

-- Rolle PT-DLR, Verwaltung des Clusters:
-- - Anlegen von KA_DBs
-- - Lschen von KA_DBs
CREATE ROLE pt_dlr NOLOGIN
    NOSUPERUSER INHERIT CREATEDB NOCREATEROLE;
GRANT waska_admin TO pt_dlr; 
GRANT waska_role_adm TO pt_dlr; 

-- Rolle Admin-KA, Elternrolle fr die Rollen in den einzelnen KA zur 
-- Verwaltung einer KA_DB:
-- - Anlegen von Nutzern in einer KA
-- - Lschen von Nutzern in einer KA
-- - Passworte von Nutzern zurcksetzen.
CREATE ROLE admin_ka NOLOGIN
    NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE;
GRANT waska_admin TO admin_ka;

-- Wartungsdatenbank
CREATE database waska_maintenance WITH TEMPLATE = template0 ENCODING = 'UTF8';
REVOKE ALL ON DATABASE waska_maintenance FROM public;
GRANT CONNECT ON DATABASE waska_maintenance TO waska_admin;

-- ------------------------------------------------------------------------
-- Anlegen der Funktionen in der Wartungsdatenbank
--

\connect waska_maintenance postgres
CREATE LANGUAGE plpgsql;
CREATE PROCEDURAL LANGUAGE 'plpythonu' HANDLER plpython_call_handler;

-- Passwortgenerator variabler Lnge. Nutzt pwgen, um konforme Passworte zu
-- generieren
CREATE OR REPLACE FUNCTION waska_passgen(pw_length int) RETURNS text 
  LANGUAGE 'plpythonu' VOLATILE
  AS $$
    from os import popen
    from random import sample 
    pw = None
    try:
      try:
        pw = popen("/usr/bin/pwgen --ambiguous --numerals --symbols %s"
                    % pw_length)
        return pw.readline().strip()
      except:
        pop = [chr(c) for c in range(33,127+1)]
        return "".join(sample(pop, pw_length)) 
    finally:
      if pw:
        try: pw.close()
        except: pass
$$;

ALTER FUNCTION waska_passgen(int) OWNER TO pt_dlr;
REVOKE ALL ON FUNCTION waska_passgen(int) FROM public;

SET ROLE pt_dlr;

\connect waska_maintenance postgres
-- waska_init_database( agentur ) returns text
-- Initialisieren einer Datenbank, mit entsprechenden Rollen fr die KA und
-- Login-Rolle 'ka_<agentur>_adm'. Rckgabewert ist das initiale Passwort der
-- Login-Rolle 'ka_<NAME AGENTUR>_adm'. Diese Funktion steht der Rolle "PT-DLR"
-- zur Verfgung.
-- 
-- Die Datenbank muss vorher angelegt sein, entsprechend muss auch die
-- Prfung auf korrekten Namen vorher stattfinden. (CREATE DATABASE kann
-- nicht in einem Skript aufgerufen werden)
CREATE OR REPLACE FUNCTION waska_init_database (
        agency varchar(52)
  ) RETURNS text LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE
    db_name   varchar(64);
    adm_group varchar(64);
    adm_user  varchar(64);
    cm_group  varchar(64);
    pb_group  varchar(64);
    pw        varchar(13);
  BEGIN
    -- Gruppen fuer die KA
    adm_group := quote_ident('ka_' || agency || '_admin_ka');
    cm_group :=  quote_ident('ka_' || agency || '_cm_ka');
    pb_group :=  quote_ident('ka_' || agency || '_pb_ka');
    EXECUTE 'CREATE ROLE ' || adm_group
        || ' NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE';
    EXECUTE 'GRANT admin_ka TO ' || adm_group;
    EXECUTE 'CREATE ROLE ' || cm_group
        || ' NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE';
    EXECUTE 'CREATE ROLE ' || pb_group
        || ' NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE';
    EXECUTE 'GRANT ' || adm_group || ' TO ' || pb_group;

    -- Administratives Konto (Besitzer der DB) einrichten
    db_name := quote_ident('ka_' || agency || '_db');
    adm_user := quote_ident('ka_' || agency || '_adm');
    pw := waska_passgen(12);
    EXECUTE 'CREATE ROLE ' || adm_user
        || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
        || ' PASSWORD ' || quote_literal(pw);
    EXECUTE 'REVOKE ALL ON DATABASE ' || db_name || ' FROM PUBLIC';
    EXECUTE 'ALTER DATABASE ' || db_name
            || ' OWNER TO ' || adm_user;
    EXECUTE 'GRANT ' || adm_group || ' TO ' || adm_user;

    RETURN pw::text;
  END;
$$ ;
REVOKE ALL ON FUNCTION waska_init_database( varchar ) FROM public;
GRANT EXECUTE ON FUNCTION waska_init_database( varchar ) TO pt_dlr;

-- Lschen einer Datenbank fr KA-DB
-- - Lschen aller Konten fr KA (stehen in Tabelle der KA)
-- - Lschen des administrativen Kontos
-- - Lschen der Gruppen fr KA
-- - Lschen der Datenbank

-- -----------------------------------------------------------------------
-- Template-Datenbank fr Kompetenzagenturen
CREATE DATABASE waska_template WITH TEMPLATE = template0 ENCODING = 'UTF8';
REVOKE ALL ON DATABASE waska_template FROM public;
\connect waska_template postgres
CREATE LANGUAGE plpgsql;
CREATE PROCEDURAL LANGUAGE 'plpythonu' HANDLER plpython_call_handler;

-- Passwortgenerator variabler Lnge
-- Passwortgenerator variabler Lnge. Nutzt pwgen, um konforme Passworte zu
-- generieren
CREATE OR REPLACE FUNCTION waska_passgen(pw_length int) RETURNS text 
  LANGUAGE 'plpythonu' VOLATILE
  AS $$
    from os import popen
    from random import sample 
    pw = None
    try:
      try:
        pw = popen("/usr/bin/pwgen --ambiguous --numerals --symbols %s"
                    % pw_length)
        return pw.readline().strip()
      except:
      	pop = [chr(c) for c in range(33,127+1)]
        return "".join(sample(pop, pw_length)) 
    finally:
      if pw:
        try: pw.close()
        except: pass
$$;

ALTER FUNCTION waska_passgen(int) OWNER TO waska_role_adm;
REVOKE ALL ON FUNCTION waska_passgen(int) FROM public;


-- Funktionen zur Benutzerverwaltung mit Rechten der Rolle waska_role_adm
-- anlegen (und spter ausfhren).
SET ROLE waska_role_adm;


-- waska_create_role( nutzer, gruppenrolle, agentur ) returns text 
-- Anlegen einer Login-Rolle 'ka_<agentur>_<nutzer>' in der gruppenrolle:
-- 'CM-KA' oder 'Admin-KA'. Es wird anhand der Namensrume berprft, ob die
-- aufrufende Rolle berechtigt ist, fr die angegebene KA Nutzer anzulegen.
-- Rckgabewert ist die Loginrolle und das initiale Passwort.
CREATE OR REPLACE FUNCTION waska_create_role (
        username varchar(64),
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS RECORD LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    db_group varchar(64);
    pb_group varchar(64);
    pw       varchar(9);
    result   RECORD;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to create role %s in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      IF lower(grouprole) = 'admin-ka' THEN
        db_group := quote_ident('ka_' || agency || '_admin_ka');
      ELSIF lower(grouprole) = 'pb-ka' THEN
        db_group := quote_ident('ka_' || agency || '_pb_ka');
      ELSE
        db_group := quote_ident('ka_' || agency || '_cm_ka');
      END IF;
      pw := waska_passgen(8);
      EXECUTE 'CREATE ROLE ' || db_user 
          || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
          || ' PASSWORD ' || quote_literal(pw);
      EXECUTE 'GRANT ' || db_group || ' TO ' || db_user;
      SELECT INTO result db_name::varchar(64), db_user::varchar(64), pw::varchar(9);
      RETURN result;
    END IF;

  END;
$$ ;
REVOKE ALL ON FUNCTION waska_create_role( varchar, varchar, varchar ) 
    FROM public;
GRANT EXECUTE ON FUNCTION waska_create_role( varchar, varchar, varchar ) 
    TO admin_ka;

-- waska_revoke_connect( nutzer, agentur ) returns boolean
-- Zurckziehen der CONNECT-Berechtigung des Nutzers fr die DB der Agentur.
-- Es wird anhand der Namensrume berprft, ob die aufrufende Rolle 
-- berechtigt ist, fr die angegebene KA Berechtigungen zurckzuziehen.
-- OWNER muss beim Anlegen der Datenbank auf den DB-Owner umgesetzt werden:
-- ALTER FUNCTION waska_revoke_role(varchar, varchar) OWNER TO ka_:FKZ:_adm;
CREATE OR REPLACE FUNCTION waska_revoke_connect (
        username varchar(64),
        agency varchar(52)
  ) RETURNS boolean LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to drop role % in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      EXECUTE 'REVOKE CONNECT ON DATABASE ' || db_name
          || ' FROM ' || db_user ;
      RETURN true;
    END IF;
    RETURN false;
  END;
$$ ;
REVOKE ALL ON FUNCTION waska_revoke_connect( varchar, varchar ) FROM public;
GRANT EXECUTE ON FUNCTION waska_revoke_connect( varchar, varchar ) TO waska_role_adm;


-- waska_drop_role( nutzer, agentur ) returns boolean 
-- Lschen der Login-Rolle
-- 'ka_<agentur>_<nutzer>'. Es wird anhand der Namensrume berprft, ob die
-- aufrufende Rolle berechtigt ist, fr die angegebene KA Nutzer zu Lschen.
CREATE OR REPLACE FUNCTION waska_drop_role (
        username varchar(64),
        agency varchar(52)
  ) RETURNS boolean LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_user       varchar(128);
    status_revoke boolean;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to drop role % in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_user := quote_ident('ka_' || agency || '_' || username);
      SELECT waska_revoke_connect(username, agency) INTO status_revoke;
      IF status_revoke
      THEN
        EXECUTE 'DROP ROLE ' || db_user ;
        RETURN true;
      END IF;
    END IF;
    RETURN false;
  END;
$$ ;
REVOKE ALL ON FUNCTION waska_drop_role( varchar, varchar ) FROM public;
GRANT EXECUTE ON FUNCTION waska_drop_role( varchar, varchar ) 
    TO admin_ka;

-- waska_reset_password ( nutzer, agentur ) returns text 
-- Setzen eines neuen Passworts fr die Login-Rolle 'ka_<agentur>_<nutzer>'. 
-- Es wird anhand der Namensrume berprft, ob die aufrufende Rolle 
-- berechtigt ist, fr die angegebene KA Nutzerpassworte zurckzusetzen. 
-- Rckgabewert ist das neue Passwort der Login-Rolle.
CREATE OR REPLACE FUNCTION waska_reset_password (
        name varchar(64),
        agency varchar(52)
  ) RETURNS text LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    pw       varchar(9);
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to reset password for role % in agency % by %.',
          quote_literal(name), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || name);
      pw := waska_passgen(8);
      EXECUTE 'ALTER USER ' || db_user || ' WITH PASSWORD ' 
          || quote_literal(pw);
      RETURN pw::text;
    END IF;
    RETURN ''::text;
  END;
$$ ;
REVOKE ALL ON FUNCTION waska_reset_password( varchar, varchar ) FROM public;
GRANT EXECUTE ON FUNCTION waska_reset_password( varchar, varchar ) 
    TO admin_ka;

-- Make the database a template database.
\connect waska_maintenance postgres
UPDATE pg_database SET datistemplate = true WHERE datname = 'waska_template';
UPDATE pg_database SET datallowconn = false WHERE datname = 'waska_template';

--
-- ENDE Initialisierung des WASKA-Clusters
-- ------------------------------------------------------------------------
